diff --git a/common/common.c b/common/common.c
index c163e09..6074f3b 100644
--- a/common/common.c
+++ b/common/common.c
@@ -631,9 +631,9 @@ void x264_picture_alloc( x264_picture_t *pic, int i_csp, int i_width, int i_heig
     pic->i_qpplus1 = 0;
     pic->img.i_csp = i_csp;
     pic->img.i_plane = 3;
-    pic->img.plane[0] = x264_malloc( 3 * i_width * i_height / 2 );
-    pic->img.plane[1] = pic->img.plane[0] + i_width * i_height;
-    pic->img.plane[2] = pic->img.plane[1] + i_width * i_height / 4;
+    pic->img.plane[0] = x264_malloc( i_width * i_height );
+    pic->img.plane[1] = x264_malloc( i_width * i_height / 4 );
+    pic->img.plane[2] = x264_malloc( i_width * i_height / 4 );
     pic->img.i_stride[0] = i_width;
     pic->img.i_stride[1] = i_width / 2;
     pic->img.i_stride[2] = i_width / 2;
@@ -645,6 +645,8 @@ void x264_picture_alloc( x264_picture_t *pic, int i_csp, int i_width, int i_heig
 void x264_picture_clean( x264_picture_t *pic )
 {
     x264_free( pic->img.plane[0] );
+    x264_free( pic->img.plane[1] );
+    x264_free( pic->img.plane[2] );
 
     /* just to be safe */
     memset( pic, 0, sizeof( x264_picture_t ) );
diff --git a/common/ppc/pixel.c b/common/ppc/pixel.c
index 5553684..7427be5 100644
--- a/common/ppc/pixel.c
+++ b/common/ppc/pixel.c
@@ -1636,7 +1636,7 @@ static int pixel_ssd_8x8_altivec ( uint8_t *pix1, int i_stride_pix1,
  ****************************************************************************/
 static int x264_pixel_var_16x16_altivec( uint8_t *pix, int i_stride )
 {
-    DECLARE_ALIGNED_16(uint32_t sum);
+    DECLARE_ALIGNED_16(uint32_t sum_tab[4]);
     DECLARE_ALIGNED_16(uint32_t sqr);
 
     LOAD_ZERO;
@@ -1652,51 +1652,52 @@ static int x264_pixel_var_16x16_altivec( uint8_t *pix, int i_stride )
 
         pix += i_stride;
     }
-    sum_v = vec_add( sum_v, vec_sld( sum_v, sum_v, 8 ) );
+    sum_v = (vec_s32_t)vec_sums( (vec_s32_t)sum_v, zero_s32v );
     sqr_v = vec_add( sqr_v, vec_sld( sqr_v, sqr_v, 8 ) );
-    sum_v = vec_add( sum_v, vec_sld( sum_v, sum_v, 4 ) );
     sqr_v = vec_add( sqr_v, vec_sld( sqr_v, sqr_v, 4 ) );
-    vec_ste(sum_v, 0, &sum);
+    vec_ste(sum_v, 12, sum_tab);
     vec_ste(sqr_v, 0, &sqr);
 
+    uint32_t sum = sum_tab[3];
     uint32_t var = sqr - (sum * sum >> 8);
     return var;
 }
 
 static int x264_pixel_var_8x8_altivec( uint8_t *pix, int i_stride )
 {
-    DECLARE_ALIGNED_16(uint32_t sum);
+    DECLARE_ALIGNED_16(uint32_t sum_tab[4]);
     DECLARE_ALIGNED_16(uint32_t sqr);
 
     LOAD_ZERO;
     vec_u32_t sqr_v = zero_u32v;
     vec_u32_t sum_v = zero_u32v;
 
-    vec_u8_t perm0 = vec_lvsl( 0, pix );
-    vec_u8_t perm1 = vec_lvsl( 0, pix+i_stride );
+    static const vec_u8_t perm_tab[] = {
+        CV(0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,  /* pix=mod16, i_stride=mod16 */
+           0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17),
+        CV(0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,  /* pix=mod16, i_stride=mod8  */
+           0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F),
+    };
+    vec_u8_t perm = perm_tab[ ((i_stride & 8) >> 3) ];
 
     int y;
     for( y = 0; y < 8; y+=2 )
     {
         vec_u8_t pix0_v = vec_ld(0, pix);
         vec_u8_t pix1_v = vec_ld(i_stride, pix);
-        pix0_v = vec_perm(pix0_v, pix0_v, perm0);
-        pix1_v = vec_perm(pix1_v, pix1_v, perm1);
-
-        vec_u8_t pix_v = vec_mergeh(pix0_v, pix1_v);
-
+        vec_u8_t pix_v = vec_perm(pix0_v, pix1_v, perm);
         sum_v = vec_sum4s(pix_v, sum_v);
         sqr_v = vec_msum(pix_v, pix_v, sqr_v);
 
         pix += i_stride<<1;
     }
-    sum_v = vec_add( sum_v, vec_sld( sum_v, sum_v, 8 ) );
+    sum_v = (vec_s32_t)vec_sums( (vec_s32_t)sum_v, zero_s32v );
     sqr_v = vec_add( sqr_v, vec_sld( sqr_v, sqr_v, 8 ) );
-    sum_v = vec_add( sum_v, vec_sld( sum_v, sum_v, 4 ) );
     sqr_v = vec_add( sqr_v, vec_sld( sqr_v, sqr_v, 4 ) );
-    vec_ste(sum_v, 0, &sum);
+    vec_ste(sum_v, 12, sum_tab);
     vec_ste(sqr_v, 0, &sqr);
 
+    uint32_t sum = sum_tab[3];
     uint32_t var = sqr - (sum * sum >> 6);
     return var;
 }